### Nested Logit model estimation

### Create the data array from the existing dataset
rm(list=ls())
setwd("C:/Users/Samuri/Documents/Project1")
library(maxLik)

Data <- read.csv("alldata_sex3.csv",header=T)
## Count the number of data rows
hh <- nrow(Data)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
b0 <- numeric(14) 

#Data$Age <- as.numeric(Data[2])
#Data$Male <- as.numeric(Data[3])
#Data$FareTrain <- as.numeric(Data[11])

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  ### declare the parameters###
  ## Alternative specific constants
  b1 <- x[1]
  b2 <- x[2]
  b3 <- x[3]
  b4 <- x[4]
  
  ## Time 
  d1 <- x[5]
  
  ## Fare
  c1 <- x[6]
  c2 <- x[7]
  
  ## Access
  a1 <- x[8]
  a2 <- x[9]
  
  # Gender
  g1 <- x[10]
  g2 <- x[11]
  g3 <- x[12]
  g4 <- x[13]
  
  
  
  
  ##Scale parameter
  pa1 <- x[14] 
  
  #pa2 <- 1
  
  
  ## declare the log-likelihood variable, set value to 0
  LL = 0
  
  ### For this choice problem, we consider the following 5 modes
  ## ??(train)  - (public)
  ## バス(bus)    - (public)
  ## 自動?(car)  - (private)
  ## 自転?(bike) - (private)
  ## 徒歩(walk)   - (private)
  
  ## calculate the utility function: :introduce the desired explanatory variables in the function
  # time           # fare      # constant
  #+ d2 * Data$FareTrain/100
  # + d2 * Data$FareBus/100
  # + a1 * Data$Age
  # + a3 * Data$Age
  # + g1 * Data$Male
  #+ g3 * Data$Male
  # + g4 * Data$Male
  # + w3*Data$Weekdays
  # + w4*Data$Weekdays
  # + p2*Data$Peak
  # + p3*Data$Peak
  
  #  + de2*Data$DepartureTime
  #  + de3 * Data$DepartureTime
  
  #  + de1 * Data$DepartureTime
  #  + d3 * Data$DistanceBus/10
  # + d3 * Data$DistanceCar/10000
  # + d3 * Data$DistanceBike/10000
  # + d3 * Data$DistanceWalk/10000
  # + wd1*Data$WorkDummy
  # + wd2*Data$WorkDummy
  # + wd4*Data$WorkDummy
  
  train    <- exp(d1 * Data$TimeBus + g1 * Data$Gendertrain + c1 * Data$FareTrain + a1*Data$AccessTrain    +b1*matrix(1,nrow =hh,ncol=1))
  bus   <- exp(d1 * Data$TimeBus + c2 * Data$FareBus + a2*Data$AccessBus   +b2*matrix(1,nrow =hh,ncol=1))
  car  <- exp(d1  * Data$TimeCar + g2*Data$Gendercar+b3*matrix(1,nrow =hh,ncol=1))
  bike    <- exp(d1 * Data$TimeBike + g3*Data$Genderbike +b4*matrix(1,nrow =hh,ncol=1))
  walk   <- exp(d1 * Data$TimeWalk + g4 * Data$Genderwalk  )
  
  ### Construct nests.First, calculate public or private choice probabilityP(LV1)
  ## setting of log-sum variables
  logsum.public  <- log( ( (train + bus )!=0) * (train + bus ) + ((train + bus )==0))
  logsum.private <- log( ( ( car+bike + walk)!=0) * (car+bike + walk) + ((car+bike + walk )==0))
  
  ##Calculate public or private choice probability with these log-sum variables
  nume.public   <- (logsum.public != 0)*exp(pa1 * logsum.public) + (logsum.public == 0)
  nume.private  <- (logsum.private != 0)*exp(pa1 * logsum.private) + (logsum.private == 0)
  
  deno <- nume.public + nume.private
  #deno <- nume.motorized + bike + walk
  P.public  <- nume.public  / deno
  P.private <- nume.private / deno
  #Pcar <- Data$ModeAvailableCar * (car / deno)
  #Pbike <- Data$ModeAvailableBike * (bike / deno)
  #Pwalk <- Data$ModeAvailableWalk * (walk / deno)
  
  
  
  ### Next, calculate conditional probability.P(LV2|LV1)
  ## conditional probability under public choice
  deno.public     <- train + bus 
  P.train.public  <- train / ((deno.public!=0)*deno.public + (deno.public==0))
  P.bus.public   <- bus   / ((deno.public!=0)*deno.public + (deno.public==0))
 
  ## conditional probability under private choice
  deno.private    <- car+bike + walk
  #P.car.private   <- car   / ((deno.private!=0)*deno.private + (deno.private==0))
  P.car.private  <- car  / ((deno.private!=0)*deno.private + (deno.private==0))
  P.bike.private  <- bike  / ((deno.private!=0)*deno.private + (deno.private==0))
  P.walk.private  <- walk  / ((deno.private!=0)*deno.private + (deno.private==0))
  
  
  ### Finally, calculate joint probability. P(LV1, LV2) = P(LV2|LV1)*P(LV1)
  P.train <- P.train.public * P.public
  P.bus   <- P.bus.public   * P.public
  P.car   <- P.car.private  * P.private
  P.bike  <- P.bike.private * P.private
  P.walk  <- P.walk.private * P.private
  
  ## Avoid problems stemming from choice probabilities becoming zero.
  P.train <- (P.train!=0)*P.train + (P.train==0)
  P.bus   <- (P.bus!=0)  *P.bus   + (P.bus==0)
  P.car   <- (P.car!=0)  *P.car   + (P.car==0)
  P.bike  <- (P.bike!=0) *P.bike  + (P.bike==0)
  P.walk  <- (P.walk!=0) *P.walk  + (P.walk==0)
  #Pbike <- (Pbike !=0) * Pbike + (Pbike == 0)
  #Pwalk   <- (Pwalk !=0) * Pwalk     + (Pwalk == 0)
  #Pcar   <- (Pcar !=0) * Pcar   + (Pcar == 0)
  
  ## Choice results
  C.train   <- Data$Mode =="rail"
  C.bus     <- Data$Mode =="bus"
  C.car     <- Data$Mode =="car"
  C.bike    <- Data$Mode =="bike"
  C.walk    <- Data$Mode =="walk"
  
  ## Calculate the Log-likelihood function
  LL <- colSums(C.train*log(P.train) + C.bus*log(P.bus) + C.car  *log(P.car) + C.bike*log(P.bike) + C.walk *log(P.walk))
  
}

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr, method = "BFGS", hessian = TRUE, control=list(fnscale=-1))

## Parmeter estimation、Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)
